/**@name LIBRARY: unLHCLoggingDBParser.ctl

@author: Josef Hofer (EN-ICE-SCD)

Creation Date: 04/09/2014

Purpose:
This library contains config file related functions for the unLHCLoggingDB component.

Usage: Public

PVSS manager usage: CTRL, UI

Constraints:
	. PVSS version: 3.11
	. operating system: Linux.

Copyright CERN

*/

//@{

// Version of this library. (Only increment this, if we edit this file)
const string c_unLHCLoggingDBParser_LibVersion = "1.0.5";

// public constants to indicate result of parsing process
public const unsigned c_unLHCLoggingDBParser_ConfigFile_ErrorNoImport   = 0;
public const unsigned c_unLHCLoggingDBParser_ConfigFile_ReadyButConfirm = 1;
public const unsigned c_unLHCLoggingDBParser_ConfigFile_ReadyForImport  = 2;

// Allowed characters
private const dyn_char c_dcAllowedChars = makeDynChar("A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", 
        "W", "X", "Y", "Z", "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "_", "-", "+", "=", ".", ":", ";", ",", "/", " ", "(", ")", "[", "]");

/**
* Get all data drom config file and database, which is needed to register the file for logging
*
* @param   sConfigFile         input,  Pathname of the config file
* @param   &dsVariableName     output, list of VARIABLE_NAMES from config file
* @param   &dsHierarchy        output, list of HIERARCHIES from config file
* @param   &dsDescription      output, list of DESCRIPTIONS from config file
* @param   &dsElementId        output, list of ELEMENT_IDS from ELEMENTS table
* @param   &dsUnit             output, list of UNITSS from ELEMENTS table
* @param   &dsRDBSource        output, list of GROUP_NAMES from ELEMENTS table
* @param   &dsDataCategory     output, list of data categories
* @param   &bDataCategoryManualInput     output, Manual input of data category is required
* @param   &dsErrors           output, list of errors
* @param   &dsWarnings         output, list of warnings
*
* @return  bSuccess            output, true=success,false=error
*/
public unsigned unLHCLoggingDBParser_getImportData(
        const string sConfigFile,
        dyn_string &dsVariableName,
        dyn_string &dsHierarchy,
        dyn_string &dsDescription,
        dyn_string &dsElementId,
        dyn_string &dsUnit,
        dyn_string &dsRDBSource,
        dyn_string &dsDataCategoryId,
        bool &bDataCategoryManualInput,
        dyn_string &dsErrors,
        dyn_string &dsWarnings)
{
    dyn_string dsDpe;
    dyn_string dsAlias;
    dyn_string dsDataCategory;

    dynClear(dsVariableName);
    dynClear(dsHierarchy);
    dynClear(dsDescription);
    dynClear(dsElementId);
    dynClear(dsUnit);
    dynClear(dsRDBSource);
    dynClear(dsErrors);
    dynClear(dsWarnings);

    bool bSuccess = true;
    bSuccess &= unLHCLoggingDBParser_parseConfigFile(sConfigFile, dsVariableName, dsDpe, dsAlias, dsHierarchy, dsDescription, dsDataCategory, dsErrors, dsWarnings);
    dyn_string dsDpeBackup = dsDpe;
    bSuccess &= unLHCLoggingDBParser_checkVarNames(dsVariableName, dsErrors, dsWarnings);
    bSuccess &= unLHCLoggingDBParser_checkHierarchies(dsHierarchy, dsErrors, dsWarnings);
    bSuccess &= unLHCLoggingDBParser_replaceChars(dsDescription, dsWarnings);
    bSuccess &= unLHCLoggingDBParser_doWhiteListCheck(dsDpe, dsAlias, dsVariableName, dsDescription, dsHierarchy, dsErrors, dsWarnings);
    bSuccess &= unLHCLoggingDBParser_getDataCategoryId(dsDpe, dsAlias, dsDataCategory, dsDataCategoryId, bDataCategoryManualInput, dsErrors, dsWarnings);
    bSuccess &= unLHCLoggingDBParser_checkDpConsistency (dsDpe, dsAlias, dsErrors, dsWarnings);
    bSuccess &= unLHCLoggingDBParser_getElementData(dsDpe, dsDpeBackup, dsAlias, dsElementId, dsUnit, dsRDBSource, dsErrors, dsWarnings);

    // If elements are already registered user has to confirm to be allowed to proceed with the import
    bool bConfirmBeforeImport = unLHCLoggingDBParser_checkIfVariablesAlreadyRegistered(dsDpe, dsAlias, dsElementId, dsVariableName, dsErrors, dsWarnings);

    if (bSuccess && bConfirmBeforeImport) { return c_unLHCLoggingDBParser_ConfigFile_ReadyForImport; }
    if (bSuccess && !bConfirmBeforeImport) { return c_unLHCLoggingDBParser_ConfigFile_ReadyButConfirm; }
    return c_unLHCLoggingDBParser_ConfigFile_ErrorNoImport;
}

/**
* Get data from config file and resolve dp names.
*
* This function is needed to do the full consictency check
*
* @param   sConfigFile         input,  Pathname of the config file
* @param   &dsVariableName     output, variable names from config file
* @param   &dsDpe              output, resolved dpe names from config file
* @param   &dsHierarchy        output, hierarchies from config file
* @param   &dsDescription      output, descriptions from config file
* @param   &dsDataCategory     output, data category names from config file
*
* @return  bSuccess            output, true=success,false=error
*/
public bool unLHCLoggingDBParser_getConfigFileData(
        const string sConfigFile,
        dyn_string &dsVariableName,
        dyn_string &dsDpe,
        dyn_string &dsHierarchy,
        dyn_string &dsDescription,
        dyn_string &dsDataCategory)
{
    dyn_string dsAlias;
    dyn_string dsErrors;
    dyn_string dsWarnings;

    dynClear(dsVariableName);
    dynClear(dsDpe);
    dynClear(dsHierarchy);
    dynClear(dsDescription);
    dynClear(dsDataCategory);

    bool bSuccess = true;
    bSuccess &= unLHCLoggingDBParser_parseConfigFile(sConfigFile, dsVariableName, dsDpe, dsAlias, dsHierarchy, dsDescription, dsDataCategory, dsErrors, dsWarnings);
    bSuccess &= unLHCLoggingDBParser_replaceChars(dsDescription, dsWarnings);
    bSuccess &= unLHCLoggingDBParser_checkDpConsistency (dsDpe, dsAlias, dsErrors, dsWarnings);

    return bSuccess;
}

/**
* Parse config file and store data into dyn_strings
*
* @param   sFileName       input,  Pathname of the config file
* @param   &dsVariableName output, list of variable names
* @param   &dsDpe          output, list of dpe names
* @param   &dsAlias        output, list of dpe aliases
* @param   &dsHierarchy    output, list of hierarchies
* @param   &dsDescription  output, list of desciptions
* @param   &dsDataCategory output, list of data categories
* @param   &dsErrors       output, list of errors
* @param   &dsWarnings     output, list of warnings
*
* @return                  output, true=success,false=error
*/
private bool unLHCLoggingDBParser_parseConfigFile(
        const string sFileName,
        dyn_string &dsVariableName,
        dyn_string &dsDpe,
        dyn_string &dsAlias,
        dyn_string &dsHierarchy,
        dyn_string &dsDescription,
        dyn_string &dsDataCategory,
        dyn_string &dsErrors,
        dyn_string &dsWarnings)

{
    // Check file format
    if (strpos(strtoupper(sFileName), ".XML") >= 0) {
        return unLHCLoggingDBParser_parseXmlConfigFile(sFileName, dsVariableName, dsDpe, dsAlias, dsHierarchy, dsDescription, dsDataCategory, dsErrors, dsWarnings);
    }
    return unLHCLoggingDBParser_parseTextConfigFile(sFileName, dsVariableName, dsDpe, dsAlias, dsHierarchy, dsDescription, dsErrors, dsWarnings);
}

/**
* Parse Xml file and store data into dyn_strings
*
* For example config files see: unLHCLoggingDB/PVSS/data/
*
* @param   sFileName       input,  Pathname of the XML file
* @param   &dsVariableName output, list of variable names
* @param   &dsDpe          output, list of dpe names
* @param   &dsAlias        output, list of dpe aliases
* @param   &dsHierarchy    output, list of hierarchies
* @param   &dsDescription  output, list of desciptions
* @param   &dsDataCategory output, list of data categories
* @param   &dsErrors       output, list of errors
* @param   &dsWarnings     output, list of warnings
*
* @return                  output, true=success,false=error
*/
private bool unLHCLoggingDBParser_parseXmlConfigFile(
        const string sFileName,
        dyn_string &dsVariableName,
        dyn_string &dsDpe,
        dyn_string &dsAlias,
        dyn_string &dsHierarchy,
        dyn_string &dsDescription,
        dyn_string &dsDataCategory,
        dyn_string &dsErrors,
        dyn_string &dsWarnings)
{
    // open XML file
    string errMsg, errLin, errCol;
    int docum = xmlDocumentFromFile ( sFileName , errMsg , errLin , errCol );  // returns document ID, or -1 on error.
    if (docum == -1) {
        dynAppend(dsErrors, "Error: Parsing: '" + errMsg + "' Line=" + errLin + " Column=" + errCol);
        return false;
    }

    // Find root node
    int rootNode = xmlFirstChild (docum);  // returns the root node, or -1 if it does not exist
    if (rootNode == -1) {
        dynAppend(dsErrors, "Error: Root node does not exist");
        return false;
    }
    if (xmlNodeType(docum, rootNode) != XML_ELEMENT_NODE) { // root node is no element node
        do {
            rootNode = xmlNextSibling(docum, rootNode); // get next node till we find an element node
            if (rootNode == -1) {
                dynAppend(dsErrors, "Error: Next node does not exist");
                return false;
            }
        } while (xmlNodeType(docum, rootNode) != XML_ELEMENT_NODE);
    }
    
    string sApplication, sVersion;
    unLHCLoggingDBParser_getNodeAttribute(docum, rootNode, "application", sApplication);
    unLHCLoggingDBParser_getNodeAttribute(docum, rootNode, "version", sVersion);    
    if(sApplication != "unLHCLoggingDB" || sVersion!="1") {
        dynAppend(dsErrors, "Error: Wrong config file format");
        return false;
    }
    
    // Retrieve child nodes
    dyn_int childNode;
    if (xmlChildNodes(docum, rootNode, childNode) == -1) { // 0 on success, -1 on error.
        dynAppend(dsErrors, "Error: While reading child nodes");
        return false;
    } 
  
    bool bSuccess = true;
    
    // real element index, cause loop index also counts non-element-nodes 
    // (e.g. comments) and we don't want holes in the output arrays
    int id = 0;
   
    int numberOfChilds = dynlen(childNode); 
    for (int i = 1; i <= numberOfChilds; i++) { // Iterate for each child
        
        // Only process element nodes, because we don't care about comment nodes etc.
        if (xmlNodeType(docum, childNode[i]) != XML_ELEMENT_NODE) { continue; };
        
        // Only process node name == 'configData', because we don't care about other ones
        if (xmlNodeName(docum, childNode[i]) != "configData") { continue; };   
        
        // Retrieve signal nodes
        dyn_int nodes;
        if (xmlChildNodes(docum, childNode[i], nodes) == -1) { // 0 on success, -1 on error.
            dynAppend(dsErrors, "Error: While reading child nodes");
            return false;
        }   
       
        // process signal nodes
        int numberOfNodes = dynlen(nodes);
        for (int j = 1; j <= numberOfNodes; j++) { // Iterate for each node
            unLHCLoggingDBUtils_animateProgressBar(100 * j / numberOfNodes);

            // Only process element nodes, because we don't care about comment nodes etc.
            if (xmlNodeType(docum, nodes[j]) != XML_ELEMENT_NODE) { continue; };
                
            // Only process node name == signal, because we don't care about other ones
            if (xmlNodeName(docum, nodes[j]) != "signal") { continue; };
              
            id++;

            // Get node attribute 'dpe' (optional)
            unLHCLoggingDBParser_getNodeAttribute(docum, nodes[j], "dpe", dsDpe[id]);

            // Get node attribute 'alias' (optional)
            unLHCLoggingDBParser_getNodeAttribute(docum, nodes[j], "alias", dsAlias[id]);

            // Get node attribute 'name' (mandatory)
            if (unLHCLoggingDBParser_getNodeAttribute(docum, nodes[j], "name", dsVariableName[id]) == false) {
                dynAppend(dsErrors, "Error: Alias '"+dsAlias[id]+"' Dpe '"+dsDpe[id]+"' has no attribute: 'Name'");
                bSuccess = false;
            }

            // Get node attribute 'hierarchy' (optional)
            if (unLHCLoggingDBParser_getNodeAttribute(docum, nodes[j], "hierarchy", dsHierarchy[id]) == false) {
                dynAppend(dsWarnings, "Warning: Alias '"+dsAlias[id]+"' Dpe '"+dsDpe[id]+"' has no attribute: 'Hierarchy'");
            }

            // Get node attribute 'description' (optional)
            unLHCLoggingDBParser_getNodeAttribute(docum, nodes[j], "description", dsDescription[id]);

            // Get node attribute 'dataCategory' (optional)
            unLHCLoggingDBParser_getNodeAttribute(docum, nodes[j], "dataCategory", dsDataCategory[id]);           
        }                     
    }    

    // no elements found in file ?
    if (id == 0) { return false; }

    return bSuccess;
}

/**
* Get value of xml node attribute and check if attribute is missing or has no value
*
* @param   doc         input, xml document id
* @param   node        input, node number
* @param   attr        input, attribute name
* @param   &value      output, attribute value
*
* @return              output, true=success,false=error
*/
private bool unLHCLoggingDBParser_getNodeAttribute (const unsigned doc, const unsigned node, const string attr, string &value)
{
    if (xmlGetElementAttribute(doc, node, attr, value) == -1) { // Attribute not found
        return false;
    } else { // Attribute found
        if (value == "") { // Attribute has no value
            return false;
        }
    }
    return true;
}

/**
* Parse text file and store data into dyn_strings
*
* For example config files see: unLHCLoggingDB/PVSS/data/
*
* @param   sFileName       input,  Pathname of the XML file
* @param   &dsVariableName output, list of variable names
* @param   &dsDpe          output, list of dpe names
* @param   &dsAlias        output, list of dpe aliases
* @param   &dsHierarchy    output, list of hierarchies
* @param   &dsDescription  output, list of desciptions
* @param   &dsErrors       output, list of errors
* @param   &dsWarnings     output, list of warnings
*
* @return                  output, true=success,false=error
*/
private bool unLHCLoggingDBParser_parseTextConfigFile(
        const string sFileName,
        dyn_string &dsVariableName,
        dyn_string &dsDpe,
        dyn_string &dsAlias,
        dyn_string &dsHierarchy,
        dyn_string &dsDescription,
        dyn_string &dsErrors,
        dyn_string &dsWarnings)
{
    file config = fopen(sFileName, "r");
    if (ferror(config)) {
        fclose(config);
        dynAppend(dsErrors, "Error while opening config file: " + sFileName);
        return false;
    }

    bool bSuccess = true;
    int id = 0; // element index

    while (feof(config) == 0) { // as long as it is not at the end of the file
        // Extract one line from config file
        string sLine;
        fgets(sLine, 5000, config); // magic number to specify a max line length which is big enough to never cause problems
        sLine = strrtrim(sLine, "\n\r "); // remove carriage return, line feed and spaces at the end of the line

        // Get Dpe
        if (patternMatch("*dpe=*", sLine)) {
            if (id > 0) { // don't check on first run to avoid 'index out of range exception'
                if (dsVariableName[id] == "") { dynAppend(dsErrors, "Error: Alias '"+dsAlias[id]+"' Dpe '"+dsDpe[id]+"' has no attribute: 'Name'"); bSuccess = false; };
                if (dsHierarchy[id] == "") {dynAppend(dsWarnings, "Warning: Alias '"+dsAlias[id]+"' Dpe '"+dsDpe[id]+"' has no attribute: 'Hierarchy'"); };
            }

            id++; // create new data set
            dsDpe[id] = substr(sLine, strpos(sLine, "=") + 1);
            dsVariableName[id] = "";
            dsAlias[id] = "";
            dsDescription[id] = "";
            dsHierarchy[id] = "";
            continue;
        }

        // Get hierarchy // important: extract hierarchy before 'name' to avoid problem that 'node name' is interpreted as 'name'
        if (patternMatch("*hierarchy=*", sLine)) 	{
            string sNode = "";
            string sPattern = sLine;
            int iPos = strpos(sPattern, "node name=");
            while (iPos >= 0) {
                sPattern = substr(sPattern, iPos + 11, strlen(sPattern) - (iPos + 11));
                if (sNode == "") { sNode = substr(sPattern, 0, strlen(sPattern) - (strlen(sPattern) - strpos(sPattern, "\""))); }
                else	{ sNode = sNode + "/" + substr(sPattern, 0, strlen(sPattern) - (strlen(sPattern) - strpos(sPattern, "\""))); }
                iPos = strpos(sPattern, "node name=");
            }
            dsHierarchy[id] = sNode ;
            continue;
        }

        // Get name
        if (patternMatch("*name=*", sLine)) 	{
            dsVariableName[id] = substr(sLine, strpos(sLine, "=") + 1);
            continue;
        }

        // Get alias
        if (patternMatch("*alias=*", sLine)) 	{
            dsAlias[id] = substr(sLine, strpos(sLine, "=") + 1);
            continue;
        }

        // Get description
        if (patternMatch("*description=*", sLine)) 	{
            dsDescription[id] = substr(sLine, strpos(sLine, "=") + 1);
            continue;
        }
    }
    fclose(config);

    if ( id > 0) { // only do this check if we have found some elements
        if (dsVariableName[id] == "") { dynAppend(dsErrors, "Error: Alias '"+dsAlias[id]+"' Dpe '"+dsDpe[id]+"' has no attribute: 'Name'"); bSuccess = false; };
        if (dsHierarchy[id] == "") {dynAppend(dsWarnings, "Warning: Alias '"+dsAlias[id]+"' Dpe '"+dsDpe[id]+"' has no attribute: 'Hierarchy'"); };
        return bSuccess;
    }

    return false;
}

/**
* Check consistency between datapoint and alias and resolve it to full datapoint element name stored in dsDpe
*
* 3 possible cases:
* - dsDpe[i] is a full DPE name
* - dsAlias[i] is a DPE alias --> it will be resolved to full DPE name
* - dsAlias[i] is a DP alias and dsDpe[i] is a element name --> it will be resolved to full DPE name
*
* @param   &dsDpe          input/output, list of dpe names
* @param   &dsAlias        input/output, list of dpe aliases
* @param   &dsErrors       output, list of errors
* @param   &dsWarnings     output, list of warnings
*
* @return                  output, true=success,false=error
*/
private bool unLHCLoggingDBParser_checkDpConsistency (
        dyn_string &dsDpe,
        dyn_string &dsAlias,
        dyn_string &dsErrors,
        dyn_string &dsWarnings)
{
    bool returnValue = true;

    int iDsDpeLen = dynlen(dsDpe);
    for (int i = 1; i <= iDsDpeLen; i++) {
        unLHCLoggingDBUtils_animateProgressBar(100 * i / iDsDpeLen);

        // Get datapoint name including SystemName (e.g.: dist_1:...)
        string sDpeWithSystemName = dpSubStr(dsDpe[i], DPSUB_SYS_DP_EL); // TOTEST
        if (sDpeWithSystemName != "") {
            // If there is a system name, add it to dsDpe, if not, leave dsDpe as it is
            dsDpe[i] = sDpeWithSystemName;
        }

        // If there is no dp and no alias
        if ((dsDpe[i] == "") && (dsAlias[i] == "")) {
            dynAppend(dsErrors, "Error: Element " + i + ", neither Dpe nor Alias specified");
            returnValue = false;
        }

        // If there is a dp and no alias
        else if ((dsDpe[i] != "") && (dsAlias[i] == "")) {
            if (dpExists(dsDpe[i]) == false) {
                dynAppend(dsErrors, "Error: Dpe '" + dsDpe[i] + "' does not exist");
                dsDpe[i] = "";
                returnValue = false;
            }
        }

        // If there is no dp but an alias
        else if ((dsDpe[i] == "") && (dsAlias [i] != "")) {
            // Retrieve alias to dp
            string sTempDpe = dpAliasToName(dsAlias[i]);

            //If dp does not exist and if di DPNAME
            if (dpExists(sTempDpe) == false) {
                dynAppend(dsErrors, "Error: Alias '" + dsAlias[i] + "' does not exist");
                dsDpe[i] = "";
                returnValue = false;
            } else if (unLHCLoggingDBParser_isDpName(sTempDpe) == true) {
                dynAppend(dsErrors, "Error: Alias '" + dsAlias[i] + "' belongs to DP '" + sTempDpe + "', please specify the Dpe");
                dsDpe[i] = "";
                returnValue = false;
            } else {
                // Prepare real dpe name for registartion
                dsDpe[i] = sTempDpe;
            }
        }

        // If there is a datapoint name and an alias
        else if ((dsDpe[i] != "") && (dsAlias[i] != "")) {
            // Retrieve alias to dp
            string sTempDpe = dpAliasToName(dsAlias[i]);

            // Check if Alias is a DPname or a DPelement. It's a DPname, if the last char is a '.'
            if (unLHCLoggingDBParser_isDpName(sTempDpe) == true) {
                // Alias is a DPNAME
                if (!dpExists(sTempDpe + dsDpe[i])) { // Check if Alias + dsDpe is a valid DPE
                    dynAppend(dsErrors, "Error: Wrong Alias or Dpe specified. Alias '" + dsAlias[i] + "' resolved to DP '" + sTempDpe + "' but Dpe is '" + dsDpe[i] + "'");
                    dsDpe[i] = "";
                    returnValue = false;
                } else {
                    // Prepare real dpe name for registartion
                    dsDpe[i] = sTempDpe + dsDpe[i];
                }
            } else {
                // Alias is a DPE
                if ((dsDpe[i] != sTempDpe) || !dpExists(sTempDpe)) {
                    dynAppend(dsErrors, "Error: Alias: '" + dsAlias[i] + "' and Dpe: '" + dsDpe[i] + "' do not match");
                    dsDpe[i] = "";
                    returnValue = false;
                } else {
                    // Leave dsDpe[i] as it is because its correct
                }
            }
        }
    }
    return returnValue;
}

/**
* Checks if a string is a DP name
*
* Dot was found at last position -> it is a DP name
*
* @param   &sDp          input, DP string
* @return                output, true: it's a DP name, false: it's no DP name
*/
private bool unLHCLoggingDBParser_isDpName(string sDp)
{
    int length = strlen(sDp);
    return (strpos(sDp, ".", length - 1) >= 0);
}

/**
* Get additional data (ELEMENT_ID, UNIT, RDBSource) from ELEMENTS table
*
* @param   &dsFullDpeName  input, list of resolved Dpes
* @param   &dsDpe          input, list of Dpes
* @param   &dsAlias        input, list of Aliases
* @param   &dsElementId    output, list of ELEMENT_IDS from ELEMENTS table
* @param   &dsUnit         output, list of UNITSS from ELEMENTS table
* @param   &dsRDBSource    output, list of RDBSources from ELEMENTS table
* @param   &dsErrors       output, list of errors
* @param   &dsWarnings     output, list of warnings
*
* @return                  output, true=success,false=error
*/
public bool unLHCLoggingDBParser_getElementData(
        const dyn_string &dsFullDpeName,
        const dyn_string &dsDpe,
        const dyn_string &dsAlias,
        dyn_string &dsElementId,
        dyn_string &dsUnit,
        dyn_string &dsRDBSource,
        dyn_string &dsErrors,
        dyn_string &dsWarnings)
{
    bool bSuccess = true;

    string sMySystemDbSchema = unLHCLoggingDBSettings_getMySystemDbSchema();

    dbCommand cmd;
    string sql = "SELECT " +
                 "  TO_CHAR(element_id), " + // number is too big to store it in a unsigned long
                 "  unit, " +
                 "  group_name " +
                 "FROM " + 
                    sMySystemDbSchema + ".elements " +
                 "WHERE element_name = :dpe " +
                 "  AND event=1"; // event must be 1, otherwise the whole transfer job could block on import
   
    if(!unLHCLoggingDBSql_startCommand(cmd, sql)) { return false; }

    int len = dynlen(dsFullDpeName);
    for (int i = 1; i <= len; i++) {
        unLHCLoggingDBUtils_animateProgressBar(100 * i / len);

        dsElementId[i] = "";
        dsUnit[i] = "";
        dsRDBSource[i] = "";

        if (dsFullDpeName[i] == "") { continue; }

        mapping map;
        map["dpe"] = dsFullDpeName[i];
        
        dyn_dyn_mixed sqlResult;        
        if(!unLHCLoggingDBSql_executeCommand(cmd, map, sqlResult) ) { return false; }

        if (dynlen(sqlResult) > 0) {
            dsElementId[i] = sqlResult[1][1];
            dsUnit[i] = sqlResult[1][2];
            dsRDBSource[i] = sMySystemDbSchema + "." + sqlResult[1][3] + "HISTORY";
        } else { // Dpe does not exist in .ELEMENTS table
            dynAppend(dsErrors, "Error: Alias '"+dsAlias[i]+"' Dpe '"+dsDpe[i]+"' not found in ELEMENTS table. Check archive config and archive manager.");
            bSuccess = false;
        }
    }
 
    if(!unLHCLoggingDBSql_finishCommand(cmd)) { return false; }
   
    return bSuccess;
}

/**
* Resolve data category names to data category ids
*
* @param   &dsDataCategories   input, list of data category name / subname
* @param   &dsDataCategoryIds  output, list of data category ids
* @param   &DataCategoryManualInput  output, Manual input of data category required
* @param   &dsErrors           output, list of errors
* @param   &dsWarnings         output, list of warnings
*
* @return                  output, true=success,false=error
*/
private bool unLHCLoggingDBParser_getDataCategoryId(
        const dyn_string &dsDpe,
        const dyn_string &dsAlias,
        const dyn_string &dsDataCategory,
        dyn_string &dsDataCategoryId,
        bool &DataCategoryManualInput,
        dyn_string &dsErrors,
        dyn_string &dsWarnings)
{
    bool bSuccess = true;

    DataCategoryManualInput = (dynlen(dsDpe) > dynlen(dsDataCategory));

    // Initialise array
    dynClear(dsDataCategoryId);
    int iDsDpeLen = dynlen(dsDpe);
    for (int i = 1; i <= iDsDpeLen; i++) {
        dsDataCategoryId[i] = "";
    }

    // Get possible data categories from database
    dyn_string dsExInfo;
    dyn_dyn_mixed data = unLHCLoggingDBSql_getDataCategories(dsExInfo);
    if (dynlen(dsExInfo) > 0) {
        dynAppend(dsErrors, "Error: Getting data categories from database");
        return false;
    }

    // Resolve category names to category ids
    int iDsDataCategoryLen = dynlen(dsDataCategory);
    for (int i = 1; i <= iDsDataCategoryLen; i++) {
        if (dsDataCategory[i] == "")  {
            DataCategoryManualInput = true;
            continue;
        }

        // Check if data_category_name was found in db, if yes resolve to data_category_id, if no create error
        int iIndex = dynContains(data[4], dsDataCategory[i]);
        if (iIndex > 0) {
            dsDataCategoryId[i] = data[1][iIndex];
        } else {
            dynAppend(dsErrors, "Error: Alias '"+dsAlias[i]+"' Dpe '"+dsDpe[i]+"' data category '" + dsDataCategory[i] + "' does not exist");
            bSuccess = false;
        }
    }

    return bSuccess;
}

/**
* update Missing Data Category Ids with default value
*
* @param   &dsDataCategoryId        input/output, list of data category ids
* @param   &sDefaultDataCategoryId  output, default value
*/
public void unLHCLoggingDBParser_updateMissingDataCategoryIds(dyn_string &dsDataCategoryId, const string sDefaultDataCategoryId)
{
    int iDsDataCategoryIdLen = dynlen(dsDataCategoryId);
    for (int i = 1; i <= iDsDataCategoryIdLen; i++) {
        if (dsDataCategoryId[i] == "") {
            dsDataCategoryId[i] = sDefaultDataCategoryId;
        }
    }
}

/**
* Find and replace illegal characters (only in attribue 'descripton')
*
* This function is only needed to don't break compatibility to old logging
* because there are thousands of files that contain french accents etc.
* and we still want to be able to import them.
*
* By showing the warning we want to encourage people to create clean
* files that don't contain invalid characters.
*
* @param   &dsDescription   input/output, list of descriptions to replace
* @param   &dsWarnings      input/output, list of warnings
*
* @return          output, success
*/
public bool unLHCLoggingDBParser_replaceChars(dyn_string &dsDescription, dyn_string &dsWarnings)
{  
    int iReplacements = 0;
    
    int iDsDescriptionLen = dynlen(dsDescription);
    for (int i=1; i<=iDsDescriptionLen; i++) {       
        unLHCLoggingDBUtils_animateProgressBar(100 * i / iDsDescriptionLen);
        
        iReplacements+= unLHCLoggingDBParser_replaceCharsFromString(dsDescription[i]);                   
    }
    
    if(iReplacements > 0) dynAppend(dsWarnings, "Warning: Invalid characters in 'Description' have been automatically replaced");  
 
    return true; // for the time being we return true, because there is no reason that the replacement could fail (it also works with empty input string)             
}

/**
* Find and replace illegal characters 
*
* @param   &sText   input/output, text to replace
*
* @return           number of replaced characters
*/
private unsigned unLHCLoggingDBParser_replaceCharsFromString(string &sText)
{  
    int iCount = 0;
               
    iCount += strreplace(sText, "&", "and");
    iCount += strreplace(sText, "%", "percent");
    iCount += strreplace(sText, "->", " to ");
    iCount += strreplace(sText, ">", " upper ");
    iCount += strreplace(sText, "<", " lower ");
    iCount += strreplace(sText, "@", " at ");
    iCount += strreplace(sText, "", "c");
    iCount += strreplace(sText, "", ""); 
    iCount += strreplace(sText, "'", "");
    iCount += strreplace(sText, "`", ""); 
    iCount += strreplace(sText, "", "");
    iCount += strreplace(sText, "\"", ""); 
    iCount += strreplace(sText, "\\", "/");     
    iCount += strreplace(sText, "*", "x");
    iCount += strreplace(sText, "", "");
    iCount += strreplace(sText, "", "");
    iCount += strreplace(sText, "!", "");
    iCount += strreplace(sText, "?", "");        
    iCount += strreplace(sText, "#", "");	
    iCount += strreplace(sText, "~", "");
    iCount += strreplace(sText, "", "/");        
    iCount += strreplace(sText, "{", "(");
    iCount += strreplace(sText, "}", ")");      
    iCount += strreplace(sText, "", "1/2");
    iCount += strreplace(sText, "", "1/4");
    iCount += strreplace(sText, "", "3/4"); 
    iCount += strreplace(sText, "\x92", " ");
           
    iCount += strreplace(sText, "", "a");   
    iCount += strreplace(sText, "", "a");     
    iCount += strreplace(sText, "", "a");  
    iCount += strreplace(sText, "", "A"); 
    iCount += strreplace(sText, "", "A");   
    iCount += strreplace(sText, "", "A"); 
    iCount += strreplace(sText, "", "A"); 
    iCount += strreplace(sText, "", "A");
        
    iCount += strreplace(sText, "", "e");
    iCount += strreplace(sText, "", "e");
    iCount += strreplace(sText, "", "e");
    iCount += strreplace(sText, "", "e");          
    iCount += strreplace(sText, "", "E");  
    iCount += strreplace(sText, "", "E"); 
    iCount += strreplace(sText, "", "E"); 
    iCount += strreplace(sText, "", "E");               
      
    iCount += strreplace(sText, "", "i"); 
    iCount += strreplace(sText, "", "i");   
    iCount += strreplace(sText, "", "i");         
    iCount += strreplace(sText, "", "I");
    iCount += strreplace(sText, "", "I");
        
    iCount += strreplace(sText, "", "o");
    iCount += strreplace(sText, "", "o");
    iCount += strreplace(sText, "", "o");       
    iCount += strreplace(sText, "", "O");
        
    iCount += strreplace(sText, "", "u");
    iCount += strreplace(sText, "", "u");
    iCount += strreplace(sText, "", "u");       
    iCount += strreplace(sText, "", "U");
    iCount += strreplace(sText, "", "U");
    iCount += strreplace(sText, "", "U");
        
    iCount += strreplace(sText, "", "ae");
    iCount += strreplace(sText, "", "AE");
    iCount += strreplace(sText, "?", "oe");
    iCount += strreplace(sText, "?", "oe");

    iCount += strreplace(sText, "", "c");
    iCount += strreplace(sText, "", "C");

    iCount += strreplace(sText, "", "u");
    iCount += strreplace(sText, "", "b");

    return iCount;
}

/**
* Check parsed output for illegal characters (only check variable_name, description and hierarchy)
*
* @param   &dsDpe           input, list of Dpes
* @param   &dsAlias         input, list of Aliases
* @param   &dsVariableName  output, list of variable names
* @param   &dsDescription   output, list of descriptions
* @param   &dsHierarchy     output, list of hierarchies
*
* @param   &dsErrors        output, list of errors
* @param   &dsWarnings      output, list of warnings
*
* @return                  output, true=success,false=error
*/
private bool unLHCLoggingDBParser_doWhiteListCheck(
        const dyn_string &dsDpe,
        const dyn_string &dsAlias,
        const dyn_string &dsVariableName,
        const dyn_string &dsDescription,
        const dyn_string &dsHierarchy,
        dyn_string &dsErrors,
        dyn_string &dsWarnings)
{
    bool ret = true;
      
    int iDsDpeLen = dynlen(dsDpe);
    for (int i = 1; i <= iDsDpeLen; i++) {       
        unLHCLoggingDBUtils_animateProgressBar(100 * i / iDsDpeLen);

        string sErrMsg;

        string sVarName = dsVariableName[i];
        if ( unLHCLoggingDBParser_checkStringUsingWhiteList(sVarName, sErrMsg) == false) {
            dynAppend(dsErrors, "Error: Alias '"+dsAlias[i]+"' Dpe '"+dsDpe[i]+"' Attribute 'Name': " + sErrMsg);
            ret = false;
        }

        string sDescription = dsDescription[i];
        if ( unLHCLoggingDBParser_checkStringUsingWhiteList(sDescription, sErrMsg) == false) {
            dynAppend(dsErrors, "Error: Alias '"+dsAlias[i]+"' Dpe '"+dsDpe[i]+"' Attribute 'Description': " + sErrMsg);
            ret = false;
        }

        string sHierarchy = dsHierarchy[i];
        if ( unLHCLoggingDBParser_checkStringUsingWhiteList(sHierarchy, sErrMsg) == false) {
            dynAppend(dsErrors, "Error: Alias '"+dsAlias[i]+"' Dpe '"+dsDpe[i]+"' Attribute 'Hierarchy': " + sErrMsg);
            ret = false;
        }
    } 
   
    return ret;
}

/**
* Find illegal chars in string
*
* @param sInput    input, input string
* @param sErrorMsg output, error description
*
* @return          output, true: stirng is ok, false: string is not ok, illegal chars found
*/
private bool unLHCLoggingDBParser_checkStringUsingWhiteList(string sInput, string &sErrorMsg)
{  
    sInput = strtoupper(sInput);    // So we just have to specify upper case letters in c_dcAllowedChars   
    
    int iInputStringLen = strlen(sInput);      
    for (int i = 0; i < iInputStringLen; i++) {
        if(dynContains(c_dcAllowedChars, sInput[i]) <= 0) { 
            sErrorMsg = "Illegal char '" + sInput[i] + "' found."; 
            return false; 
        };
    }
    return true;     
}

/**
* Check names for max length and duplications
*
* @param   &dsVariableName  input, list of variable names
*
* @param   &dsErrors        output, list of errors
* @param   &dsWarnings      output, list of warnings
*
* @return                  output, true=success,false=error
*/
private bool unLHCLoggingDBParser_checkVarNames(
        const dyn_string &dsVariableName,
        dyn_string &dsErrors,
        dyn_string &dsWarnings)
{
    bool bSuccess = true;    
    dyn_string dsUniqueList;
    
    int iDsVariableNameLen = dynlen(dsVariableName);
    for (int i = 1; i <= iDsVariableNameLen; i++) {       
        unLHCLoggingDBUtils_animateProgressBar(100 * i / iDsVariableNameLen);
        
        if(strlen(dsVariableName[i])>51) { // magic constant from DB table
            dynAppend(dsErrors, "Error: Name '" + dsVariableName[i] + "' is too long");
            bSuccess = false;       
        }   
        
        string sVarName = strtoupper(dsVariableName[i]); // to find duplicates in both upper- and lowercase names       
        if( dynContains(dsUniqueList, sVarName) > 0 ) {
            // It is a duplicate
            dynAppend(dsErrors, "Error: Name '" + dsVariableName[i] + "' defined more than once");
            bSuccess = false;            
        } else {
            // It is no duplicate
            dynAppend(dsUniqueList, sVarName);      
        }
    }  
    return bSuccess;
}

/**
* Check, if parsed output contains hierarchy=ERROR or IGNORE
*
* @param   &dsHierarchy     input, list of hierarchies
*
* @param   &dsErrors        output, list of errors
* @param   &dsWarnings      output, list of warnings
*
* @return                  output, true=success,false=error
*/
private bool unLHCLoggingDBParser_checkHierarchies(
        const dyn_string &dsHierarchy,
        dyn_string &dsErrors,
        dyn_string &dsWarnings)
{
    bool bSuccess = true;    
    
    if(dynContains(dsHierarchy,"ERROR")>0) {
        dynAppend(dsErrors, "Error: DPEs with Hierarchy 'ERROR' detected");
        bSuccess = false;    
    }
    
    if(dynContains(dsHierarchy,"IGNORE")>0) {
        dynAppend(dsErrors, "Error: DPEs with Hierarchy 'IGNORE' detected");
        bSuccess = false;          
    }

    return bSuccess;
}

/**
* Check if list of variables is already registered for logging (in META_VARIABLES table)
*
* @param   &dsDpe          input, list of Dpes
* @param   &dsAlias        input, list of Aliases
* @param   &dsElementId    input, list of ELEMENT_IDS from ELEMENTS table
* @param   &dsVariableName input, list of VARIABLE_NAMES from config file
* @param   &dsErrors       output, list of errors
* @param   &dsWarnings     output, list of warnings
*
* @return                  output, true: variables are not registered, false: at least 1 variable already registered
*/
private bool unLHCLoggingDBParser_checkIfVariablesAlreadyRegistered(
        const dyn_string &dsDpe,
        const dyn_string &dsAlias,
        const dyn_string &dsElementId,
        const dyn_string &dsVariableName,
        dyn_string &dsErrors,
        dyn_string &dsWarnings)
{
    bool bSuccess = true;

    dbCommand cmd;
    string sql = "SELECT variable_name, TO_CHAR(element_id) "
                 "  FROM rdb2lhc.meta_variables "
                 " WHERE (element_id=:elementId OR variable_name=upper(:variableName)) "
                 "   AND obsolete_since_utc IS NULL";    
    
    if(!unLHCLoggingDBSql_startCommand(cmd, sql)) { return false; }

    int len = dynlen(dsElementId);
    for (int i = 1; i <= len; i++) {
        unLHCLoggingDBUtils_animateProgressBar(100 * i / len);

        if (dsElementId[i] == "" || dsVariableName[i] == "") { continue; }

        mapping map;
        map["elementId"] = dsElementId[i];
        map["variableName"] = dsVariableName[i];
        
        dyn_dyn_mixed sqlResult;        
        if(!unLHCLoggingDBSql_executeCommand(cmd, map, sqlResult, true) ) { return false; }
        
        if (dynlen(sqlResult) > 0) {             
           int iNamePos = dynContains(sqlResult[1], strtoupper(dsVariableName[i]));
           int iElementPos = dynContains(sqlResult[2], strtoupper(dsElementId[i]));
                                         
           if( (iNamePos<=0 && iElementPos<=0) ||  // Neither NAME nor ELEMENT_ID found in database
               iNamePos==iElementPos ) { // Current NAME and ELEMENT_ID combination is already registered 
               continue; // No error
           } 
                                           
           if(iNamePos>0) {
               dynAppend(dsWarnings, "Warning: Name '"+dsVariableName[i]+"' is already registered with a another Dpe");
               bSuccess = false;   
           }
               
           if(iElementPos>0) {
               dynAppend(dsWarnings, "Warning: Alias '"+dsAlias[i]+"' Dpe '"+dsDpe[i]+"' is already registered with a another 'Name'");
               bSuccess = false;   
           }       
        }
    }
    
    if(!unLHCLoggingDBSql_finishCommand(cmd)) { return false; }

    return bSuccess;
}

/**
* Create new config file for logging configuration
*
* @param   sFilePathName   input, full pathname of the file
* @param   bIsXmlFile      input, true=xml-file, false=txt-file
* @param   &dsDpe          input, list of dpe names
* @param   &dsAlias        input, list of dpe aliases
* @param   &dsName         input, list of variable names
* @param   &dsHierarchy    input, list of hierarchies
* @param   &dsDescription  input, list of desciptions
* @param   &dsDataCategory input, list of data categories name/subname
*
* @return                  output, true=success, false=error
*/
public bool unLHCLoggingDBParser_createConfigFile(
        string sFilePathName,
        bool bIsXmlFile,
        const dyn_string &dsDpe,
        const dyn_string &dsAlias,
        const dyn_string &dsName,
        const dyn_string &dsHierarchy,
        const dyn_string &dsDescription,
        const dyn_string &dsDataCategory)
{
    if(bIsXmlFile) {        
        return unLHCLoggingDBParser_createXmlConfigFile(sFilePathName, dsDpe, dsAlias, dsName, dsHierarchy, dsDescription, dsDataCategory);
    } 
    return unLHCLoggingDBParser_createTextConfigFile(sFilePathName, dsDpe, dsAlias, dsName, dsHierarchy, dsDescription, dsDataCategory);
}

/**
* Create new xml config file for logging configuration
*
* @param   sFilePathName   input, full pathname of the file
* @param   &dsDpe          input, list of dpe names
* @param   &dsAlias        input, list of dpe aliases
* @param   &dsName         input, list of variable names
* @param   &dsHierarchy    input, list of hierarchies
* @param   &dsDescription  input, list of desciptions
* @param   &dsDataCategory input, list of data categories name/subname
*
* @return                  output, true=success, false=error
*/
private bool unLHCLoggingDBParser_createXmlConfigFile(
        string sFilePathName,
        const dyn_string &dsDpe,
        const dyn_string &dsAlias,
        const dyn_string &dsName,
        const dyn_string &dsHierarchy,
        const dyn_string &dsDescription,
        const dyn_string &dsDataCategory)
{
    int docNum, rootNode, groupNode, childNode;
    
    const bool bUseDataCategory = dynlen(dsDataCategory) > 0;

    docNum = xmlNewDocument();
    xmlAppendChild(docNum, -1, XML_PROCESSING_INSTRUCTION_NODE, "xml version=\"1.0\" encoding=\"ISO 8859-1\"");

    rootNode = xmlAppendChild(docNum, -1, XML_ELEMENT_NODE, "configFile");
    xmlSetElementAttribute(docNum, rootNode, "application", "unLHCLoggingDB");
    xmlSetElementAttribute(docNum, rootNode, "version", "1");
        
    groupNode = xmlAppendChild(docNum, rootNode, XML_ELEMENT_NODE, "configData");
    xmlSetElementAttribute(docNum, groupNode, "group", "default"); 
                    
    int dsDpeLen = dynlen(dsDpe);
    for (int i = 1; i <= dsDpeLen; i++) {
        unLHCLoggingDBUtils_animateProgressBar(100 * i / dsDpeLen);            
            
        childNode = xmlAppendChild(docNum, groupNode, XML_ELEMENT_NODE, "signal");
        xmlSetElementAttribute(docNum, childNode, "dpe", dsDpe[i]);
        xmlSetElementAttribute(docNum, childNode, "alias", dsAlias[i]);
        xmlSetElementAttribute(docNum, childNode, "name", dsName[i]);
        xmlSetElementAttribute(docNum, childNode, "description", dsDescription[i]);
        xmlSetElementAttribute(docNum, childNode, "hierarchy", dsHierarchy[i]);
        if(bUseDataCategory) { // is optional        
            xmlSetElementAttribute(docNum, childNode, "dataCategory", dsDataCategory[i]);
        }
    }
    xmlDocumentToFile(docNum, sFilePathName); // TODO: error handling but return values are not documented  
    
    return true;
}

/**
* Create new text config file for logging configuration
*
* @param   sFilePathName   input, full pathname of the file
* @param   &dsDpe          input, list of dpe names
* @param   &dsAlias        input, list of dpe aliases
* @param   &dsName         input, list of variable names
* @param   &dsHierarchy    input, list of hierarchies
* @param   &dsDescription  input, list of desciptions
* @param   &dsDataCategory input, list of data categories name/subname
*
* @return                  output, true=success, false=error
*/
private bool unLHCLoggingDBParser_createTextConfigFile(
        string sFilePathName,
        const dyn_string &dsDpe,
        const dyn_string &dsAlias,
        const dyn_string &dsName,
        const dyn_string &dsHierarchy,
        const dyn_string &dsDescription,
        const dyn_string &dsDataCategory)
{          
    file configFile = fopen(sFilePathName,"w+"); 			  		  
    if (ferror(configFile)!=0) { return false; }   
        
    fputs("[_ValueArchive_0]\n", configFile);   
        
    int dsDpeLen = dynlen(dsDpe);
    for(int i=1; i<=dsDpeLen; i++) {
        unLHCLoggingDBUtils_animateProgressBar(100 * i / dsDpeLen);             
            
        fputs("\t"+"dpe="+dsDpe[i]+"\n", configFile);   
        fputs("\t\t"+"alias="+dsAlias[i]+"\n", configFile); 
        fputs("\t\t"+"name="+dsName[i]+"\n", configFile); 
        fputs("\t\t"+"description="+dsDescription[i]+"\n", configFile); 
        fputs("\t\t"+"hierarchy="+unLHCLoggingDBParser_createHierarchyString(dsHierarchy[i])+"\n", configFile); 
        fputs("\t\t"+"format=\n", configFile);
    }  
   
    fclose(configFile);  
    
    return true;
}

/*
Puts the backslash-separated hierarchy string into the form, that is needed for text file import  
*/
private string unLHCLoggingDBParser_createHierarchyString(string sRawHierarchy)
{
    string sResult;
    dyn_string dsRawHierarchy = strsplit(sRawHierarchy,"/");
    
    int iDsRawHierarchyLen = dynlen(dsRawHierarchy);
    for(int i=1; i<=iDsRawHierarchyLen; i++) {
        sResult += "<node name=\"" + dsRawHierarchy[i] + "\">";
    }
    for(int i=1; i<=iDsRawHierarchyLen; i++) {
        sResult += "</node>";  
    }
    return sResult;
}

/**
* Get set of datapoints that could be registered for logging
*
* @param   bShowInternalDPs   input, take internal DPs into account
* @param   &bShowDevices      input, take Devices into account
* @param   &bShowAlarms       input, take SystemAlarms into account
* @param   &bShowOthers       input, take Others (no Devices and SystemAlarms) into account
* @param   &dsPatterns        input, take user defined filter patterns into account
*
* @return  List of DPEs
*/
public dyn_string unLHCLoggingDBParser_getSetOfPossibleDPEs(
        bool bShowInternalDPs=true, 
        bool bShowDevices=true, 
        bool bShowAlarms=true, 
        bool bShowOthers=false,
        dyn_string dsPatterns=makeDynString())
{
    dyn_string dsArchivedDpes = unLHCLoggingDBParser_getArchivedDPEs(bShowInternalDPs);        
    if(dynlen(dsArchivedDpes) <= 0) { 
        return makeDynString(); 
    }  
    dyn_string dsFilterStage1 = unLHCLoggingDBParser_filterForDevicesAndAlarms(dsArchivedDpes, bShowDevices, bShowAlarms, bShowOthers); 
 
    return unLHCLoggingDBParser_filterForUserPatterns(dsPatterns, dsFilterStage1);   
}
 
/**
* Get set of datapoints that are using RDB Archive
*
* @param   bShowInternalDPs   input, true: take internal DPs into account, false: supress internal DPs
*
* @return  List of DPEs
*/
private dyn_string unLHCLoggingDBParser_getArchivedDPEs(bool bShowInternalDPs=true) 
{
    dyn_string dsExInfo;
    dyn_dyn_mixed ddmResult;
    
    string sMySystemDbSchema = unLHCLoggingDBSettings_getMySystemDbSchema();
    
    if(bShowInternalDPs) {
        ddmResult = unLHCLoggingDBSql_getElements(sMySystemDbSchema, dsExInfo);                  
    } else {
       ddmResult = unLHCLoggingDBSql_getElementsExceptInternalDPs(sMySystemDbSchema, dsExInfo);                  
    }  
   
    return ddmResult[1];
}

/**
* Filter list of DPEs for Devices, Alarms and Others
*
* @param   &bShowDevices      input, forward Devices into output list
* @param   &bShowAlarms       input, forward SystemAlarms into output list
* @param   &bShowOthers       input, forward Others (no Devices and SystemAlarms) into output list
*
* @return  dsResult, List of DPEs
*/
private dyn_string unLHCLoggingDBParser_filterForDevicesAndAlarms(const dyn_string &dsDPE, bool bShowDevices, bool bShowAlarms, bool bShowOthers) 
{
    dyn_string dsResult;       
    dyn_string dsDevices = unLHCLoggingDBParser_getDevices();  
    
    int len = dynlen(dsDPE);        
    for (int i=1; i<=len; i++) {
        unLHCLoggingDBUtils_animateProgressBar(100 * i / len); 
            
        if(!dpExists(dsDPE[i])) { continue; }
                
        bool append = false;
        string sType = dpTypeName(dsDPE[i]); 
                
        if(dynContains(dsDevices, sType) > 0) {
            if(bShowDevices) { append = true; };
        } else if(sType == c_unSystemAlarm_dpType) {
            if(bShowAlarms) { append = true; };    
        } else {
            if(bShowOthers) { append = true; };
        } 
        
        if (append) {
            dynAppend(dsResult, dpSubStr(dsDPE[i], DPSUB_DP_EL));                
        }                
    } 
 
    return dsResult;       
}

/**
* Get list of devices, that are known by the project
*
* @return  dsDevices, List of devices
*/
private dyn_string unLHCLoggingDBParser_getDevices()
{
    // get UNICOS devices
    dyn_string dsUnDevices = dpNames("*","_UnObjects");         
    for(int i=1; i<=dynlen(dsUnDevices);i++) {           
        dsUnDevices[i] = dpSubStr(dsUnDevices[i], DPSUB_DP);
        dsUnDevices[i] = substr(dsUnDevices[i],0,strlen(dsUnDevices[i])-strlen("_unicosInfo"));
    }  

    // get JCOP devices    
    dyn_string dsFwDevices = dpNames("*","_FwDeviceDefinition"); 
    for(int i=1; i<=dynlen(dsFwDevices);i++) {           
        dsFwDevices[i] = dpSubStr(dsFwDevices[i], DPSUB_DP);
        dsFwDevices[i] = substr(dsFwDevices[i],0,strlen(dsFwDevices[i])-strlen("_Info")); 
    } 

    // merge UNICOS and JCOP devices     
    dyn_string dsDevices; 
    dynAppend(dsDevices,dsUnDevices);
    dynAppend(dsDevices,dsFwDevices);
    dynUnique(dsDevices);   
   
    return dsDevices;   
}


/**
* Filter list of DPEs with a list of patterns and forward DPE to output, if thez match with at least one pattern
*
* @param   &dsPatterns      input, List of patterns
* @param   &dsDpe           input, List of DPEs
*
* @return  dsResult, List of DPEs that match patterns
*/
private dyn_string unLHCLoggingDBParser_filterForUserPatterns(const dyn_string &dsPatterns, const dyn_string &dsDpe)
{
    dyn_string dsResult;     
    
    int patterns = dynlen(dsPatterns);    
    if(patterns==0) {
        return dsDpe;
    }    
   
    int len = dynlen(dsDpe);
    for(int i=1; i<=len; i++) {
        unLHCLoggingDBUtils_animateProgressBar(100 * i / len);  
        for(int j=1; j<=patterns; j++) {                   
            if (patternMatch(dsPatterns[j], dsDpe[i])) {
                dynAppend(dsResult, dsDpe[i]);
                break;
            }
        } 
    }

    return dsResult;    
}

/**
* Retrieve logging config data for a list of DPEs using a user defined function
*
* @param   &dsDpe           input, List of DPEs
* @param   &sUserFunction   input, User defined config file generator function
*
* @return  dsResult, Logging config data as a dyn_string
*
* dsResult[1] = name
* dsResult[2] = description
* dsResult[3] = hierarchy
* dsResult[4] = alias
* dsResult[5] = dpe extension
*/
public dyn_string unLHCLoggingDBParser_retrieveConfigData(const string sDPE, const string sUserFunction)
{
    dyn_string dsResult;    
    
    // Call user defined function to retrieve data     
    evalScript(dsResult,
     "dyn_string main(string dpe) {" +
 				"string sName_Dpe, sDescription_Dpe, sHierarchy_Dpe,sAlias_Dpe, sExt_Dpe;" +
 				"dyn_string dsMenuTemp;" +
     "if (isFunctionDefined(\"" + sUserFunction + "\")) {" +
 				"   " + sUserFunction + "(dpe, sName_Dpe, sDescription_Dpe, sHierarchy_Dpe, sAlias_Dpe, sExt_Dpe);" +
 				"	  dynAppend(dsMenuTemp,sName_Dpe);" +	
  			"	  dynAppend(dsMenuTemp,sDescription_Dpe);" +
  			"	  dynAppend(dsMenuTemp,sHierarchy_Dpe);" +
  			"	  dynAppend(dsMenuTemp,sAlias_Dpe);" +
  			"	  dynAppend(dsMenuTemp,sExt_Dpe);" + 							 							 
     "} else {" +
 				"    dsMenuTemp = makeDynString();" +
     "} " + 
 				"return dsMenuTemp; }", makeDynString(), sDPE);    
        
    return dsResult;   
}

/* Future use for automatic config file generation triggered by a script. This feature is currently not finished and not in use */
private dyn_dyn_anytype unLHCLoggingDBParser_prepareConfigData(const dyn_string &dsDPE, const string sUserFunction, int &iErrors, int &iIgnored) 
{
    dyn_dyn_anytype ddaResult;   
    
    iErrors=0; 
    iIgnored=0;
    
    int len = dynlen(dsDPE);
    for (int i=1; i<=len; i++) {       
     
        dyn_string dsConfigData = unLHCLoggingDBParser_retrieveConfigData(dsDPE[i], sUserFunction);
                
        switch(dsConfigData[3])
        {
            case cs_unLHCLoggingDBUserLib_Error:
                iErrors++;
                break;
            case cs_unLHCLoggingDBUserLib_Ignore:
                iIgnored++;   
                break;
            default: 
                dynAppend(ddaResult[1], dsDPE[i]); // full dpe
                dynAppend(ddaResult[2], dsConfigData[1]); // name
                dynAppend(ddaResult[3], dsConfigData[2]); // desc
                dynAppend(ddaResult[4], dsConfigData[3]); // hier  
                dynAppend(ddaResult[5], dsConfigData[4]); // alias
                dynAppend(ddaResult[6], dsConfigData[5]); // extension                
        }                                    
    }      
    return ddaResult;
}

//@}
